@@ -2,7 +2,7 @@
  * PL/R - PostgreSQL support for R as a
  *	      procedural language (PL)
  *
- * Copyright (c) 2003-2010 by Joseph E. Conway
+ * Copyright (c) 2003-2013 by Joseph E. Conway
  * ALL RIGHTS RESERVED
  * 
  * Joe Conway <mail@joeconway.com>
@@ -31,7 +31,6 @@
  * plr.c - Language handler and support functions
  */
 #include "plr.h"
-#include "libpq/pqsignal.h"
 
 PG_MODULE_MAGIC;
 
@@ -49,6 +48,8 @@ static bool	plr_be_init_done = false;
 /* namespace OID for the PL/R language handler function */
 static Oid plr_nspOid = InvalidOid;
 
+int R_SignalHandlers = 1;  /* Exposed in R_interface.h */
+
 /*
  * defines
  */
@@ -176,10 +177,13 @@ static void plr_error_callback(void *arg);
 static Oid getNamespaceOidFromFunctionOid(Oid fnOid);
 static bool haveModulesTable(Oid nspOid);
 static char *getModulesSql(Oid nspOid);
-static char **fetchArgNames(HeapTuple procTup, int nargs);
 #ifdef HAVE_WINDOW_FUNCTIONS
 static void WinGetFrameData(WindowObject winobj, int argno, Datum *dvalues, bool *isnull, int *numels, bool *has_nulls);
 #endif
+static void plr_resolve_polymorphic_argtypes(int numargs,
+											 Oid *argtypes, char *argmodes,
+											 Node *call_expr, bool forValidator,
+											 const char *proname);
 
 /*
  * plr_call_handler -	This is the only visible function
@@ -288,6 +292,7 @@ PLR_CLEANUP
 
 	if(tmpdir)
 	{
+		int		rv;
 		/*
 		 * length needed = 'rm -rf ""' == 9
 		 * plus 1 for NULL terminator
@@ -297,7 +302,9 @@ PLR_CLEANUP
 		sprintf(buf, "rm -rf \"%s\"", tmpdir);
 
 		/* ignoring return value */
-		system(buf);
+		rv = system(buf);
+		if (rv != 0)
+			; /* do nothing */
 	}
 }
 
@@ -368,6 +375,11 @@ plr_init(void)
 	 */
 	atexit(plr_atexit);
 
+	/*
+	 * Stop R using its own signal handlers
+	 */
+	R_SignalHandlers = 0;
+
 	/*
 	 * When initialization fails, R currently exits. Check the return
 	 * value anyway in case this ever gets fixed
@@ -392,10 +404,6 @@ plr_init(void)
 	R_Interactive = false;
 #endif
 
-	/*
-	 * R seems to try to steal SIGINT in recent releases, so steal it back
-	 */
-	pqsignal(SIGINT, StatementCancelHandler);		/* cancel current query */
 
 	plr_pm_init_done = true;
 }
@@ -1134,11 +1142,31 @@ do_compile(FunctionCallInfo fcinfo,
 	if (!is_trigger)
 	{
 		int		i;
-		GET_ARG_NAMES;
+		bool		forValidator = false;
+		int			numargs;
+		Oid		   *argtypes;
+		char	  **argnames;
+		char	   *argmodes;
+
+		numargs = get_func_arg_info(procTup,
+									&argtypes, &argnames, &argmodes);
+
+		plr_resolve_polymorphic_argtypes(numargs, argtypes, argmodes,
+											 fcinfo->flinfo->fn_expr,
+											 forValidator,
+											 function->proname);
+
 
 		function->nargs = procStruct->pronargs;
 		for (i = 0; i < function->nargs; i++)
 		{
+			char		argmode = argmodes ? argmodes[i] : PROARGMODE_IN;
+
+			if (argmode != PROARGMODE_IN &&
+				argmode != PROARGMODE_INOUT &&
+				argmode != PROARGMODE_VARIADIC)
+				continue;
+
 			/*
 			 * Since we already did the replacement of polymorphic
 			 * argument types by actual argument types while computing
@@ -1392,10 +1420,9 @@ plr_parse_func_body(const char *body)
     SEXP	tmp;
 	int		status;
 
-	PROTECT(rbody = NEW_CHARACTER(1));
-	SET_STRING_ELT(rbody, 0, COPY_TO_USER_STRING(body));
+	PROTECT(rbody = mkString(body));
+	PROTECT(tmp = R_PARSEVECTOR(rbody, -1, &status));
 
-	tmp = PROTECT(R_PARSEVECTOR(rbody, -1, &status));
 	if (tmp != R_NilValue)
 		PROTECT(fun = VECTOR_ELT(tmp, 0));
 	else
@@ -1651,45 +1678,6 @@ plr_error_callback(void *arg)
 		errcontext("In PL/R function %s", (char *) arg);
 }
 
-/*
- * Fetch the argument names, if any, from the proargnames field of the
- * pg_proc tuple.  Results are palloc'd.
- *
- * Borrowed from src/pl/plpgsql/src/pl_comp.c
- */
-static char **
-fetchArgNames(HeapTuple procTup, int nargs)
-{
-	Datum		argnamesDatum;
-	bool		isNull;
-	Datum	   *elems;
-	int			nelems;
-	char	  **result;
-	int			i;
-
-	if (nargs == 0)
-		return NULL;
-
-	argnamesDatum = SysCacheGetAttr(PROCOID, procTup, Anum_pg_proc_proargnames,
-									&isNull);
-	if (isNull)
-		return NULL;
-
-	deconstruct_array(DatumGetArrayTypeP(argnamesDatum),
-					  TEXTOID, -1, false, 'i',
-					  &elems, NULL, &nelems);
-
-	if (nelems != nargs)		/* should not happen */
-		elog(ERROR, "proargnames must have the same number of elements as the function has arguments");
-
-	result = (char **) palloc(sizeof(char *) * nargs);
-
-	for (i=0; i < nargs; i++)
-		result[i] = DatumGetCString(DirectFunctionCall1(textout, elems[i]));
-
-	return result;
-}
-
 /*
  * getNamespaceOidFromFunctionOid - Returns the OID of the namespace for the
  * language handler function for the postgresql function with the OID equal
@@ -1852,3 +1840,56 @@ WinGetFrameData(WindowObject winobj, int argno, Datum *dvalues, bool *isnulls, i
 }
 #endif
 
+/*
+ * swiped out of plpgsql pl_comp.c
+ *
+ * This is the same as the standard resolve_polymorphic_argtypes() function,
+ * but with a special case for validation: assume that polymorphic arguments
+ * are integer, integer-array or integer-range.  Also, we go ahead and report
+ * the error if we can't resolve the types.
+ */
+static void
+plr_resolve_polymorphic_argtypes(int numargs,
+								 Oid *argtypes, char *argmodes,
+								 Node *call_expr, bool forValidator,
+								 const char *proname)
+{
+	int			i;
+
+	if (!forValidator)
+	{
+		/* normal case, pass to standard routine */
+		if (!resolve_polymorphic_argtypes(numargs, argtypes, argmodes,
+										  call_expr))
+			ereport(ERROR,
+					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+					 errmsg("could not determine actual argument "
+							"type for polymorphic function \"%s\"",
+							proname)));
+	}
+	else
+	{
+		/* special validation case */
+		for (i = 0; i < numargs; i++)
+		{
+			switch (argtypes[i])
+			{
+				case ANYELEMENTOID:
+				case ANYNONARRAYOID:
+				case ANYENUMOID:		/* XXX dubious */
+					argtypes[i] = INT4OID;
+					break;
+				case ANYARRAYOID:
+					argtypes[i] = INT4ARRAYOID;
+					break;
+#ifdef ANYRANGEOID
+				case ANYRANGEOID:
+					argtypes[i] = INT4RANGEOID;
+					break;
+#endif
+				default:
+					break;
+			}
+		}
+	}
+}
