@@ -2,7 +2,7 @@
  * PL/R - PostgreSQL support for R as a
  *	      procedural language (PL)
  *
- * Copyright (c) 2003-2010 by Joseph E. Conway
+ * Copyright (c) 2003-2013 by Joseph E. Conway
  * ALL RIGHTS RESERVED
  * 
  * Joe Conway <mail@joeconway.com>
@@ -33,7 +33,7 @@
 #ifndef PLR_H
 #define PLR_H
 
-#define PLR_VERSION		"08.03.00.13"
+#define PLR_VERSION		"08.03.00.15"
 
 #include "postgres.h"
 
@@ -44,6 +44,11 @@
 #include "windowapi.h"
 #endif
 #include "access/heapam.h"
+#if PG_VERSION_NUM >= 90300
+#include "access/htup_details.h"
+#else
+#include "access/htup.h"
+#endif
 #include "catalog/catversion.h"
 #include "catalog/pg_language.h"
 #include "catalog/pg_namespace.h"
@@ -103,6 +108,8 @@
 #endif
 #if !defined(WIN32) && !defined(WIN64)
 #include "Rinterface.h"
+#else
+extern int R_SignalHandlers;
 #endif
 #include "Rinternals.h"
 #include "Rdefines.h"
